# Importing flask module in the project is mandatory
# An object of Flask class is our WSGI application.
from flask import Flask, render_template, url_for, request
from PyLTSpice.LTSpice_RawRead import LTSpiceRawRead
from matplotlib import pyplot as plt
import numpy as np
import os
import shutil
import subprocess


# Flask constructor takes the name of
# current module (__name__) as argument.
app = Flask(__name__)

@app.route('/apidbase/flaskapi/')
# ‘/’ URL is bound with hello_world() function.
def hello_world():
    return render_template("index.html")

@app.route('/apidbase/flaskapi/loading')
# ‘/’ URL is bound with hello_world() function.
def loading():
  args = request.args
  circuit_type = args['circuit']
  param1_value = args['param1']
  param2_value = args['param2']
  new_value1 = "SYMATTR Value " + str(param1_value)
  new_value2 = "SYMATTR Value " + str(param2_value)
  if circuit_type == "lc":
      # modify lc filter; find and change:
      f=open("lc_filter_template.asc","rb")
      s=f.read()
      f.close()
      ss=s.replace(b'SYMATTR Value 147.0',bytes(new_value1, "utf-8"))
      sss=ss.replace(b'SYMATTR Value 3.320',bytes(new_value2, "utf-8"))
      shutil.copyfile('ltspice_lc.png', './scratchpad/ltspice_screenshot' + str(os.getpid()) + '.png')
  elif circuit_type == "active":
      # modify active filter; find and change:
      f=open("LT6261_Butterworth3rdOrder_template.asc","rb")
      s=f.read()
      f.close()
      ss=s.replace(b'SYMATTR Value 4.4',bytes(new_value1, "utf-8"))
      sss=ss.replace(b'SYMATTR Value 47',bytes(new_value2, "utf-8"))
      shutil.copyfile('ltspice_active.png', './scratchpad/ltspice_screenshot' + str(os.getpid()) + '.png')
  else:
      print("Unsupported circuit")

  f=open("filter1" + str(os.getpid()) + ".asc","wb")
  f.write(sss)
  f.close()
  return render_template("loading.html")

# The route() function of the Flask class is a decorator,
# which tells the application which URL should call
# the associated function.
@app.route('/apidbase/flaskapi/simulate')
def simulate_net():
    ltnetlist = "filter1" + str(os.getpid()) + ".asc"
    cmd = ['wine', '/home/relab_user/.wine/drive_c/Program Files/LTC/LTspiceXVII/XVIIx64.exe', '-Run -b -netlist']
    cmd.append(ltnetlist)
    subprocess.call(cmd, shell=False)

    LTR = LTSpiceRawRead("filter1" + str(os.getpid()) + ".raw")

    #print(LTR.get_trace_names())
    #print(LTR.get_raw_property())

    TRF = LTR.get_trace("V(vout)")
    x   = LTR.get_trace("frequency")
    #print(x)
    #print(TRF)
    fig, ax = plt.subplots(figsize=(10, 5))
    ax.plot(np.real(x) / 1000000, 20.0*np.log10(np.abs(TRF)))
    ax.xaxis.set_major_formatter(plt.FormatStrFormatter('%6.3fMHz'))
    ax.yaxis.set_major_formatter(plt.FormatStrFormatter('%4.1fdB'))
    #ax.title("AC Analysis")
    fig.canvas.manager.set_window_title('AC Analysis') 
    plt.grid()
    figname_per_worker = "./scratchpad/acgraph" + str(os.getpid()) + ".png"
    plt.savefig(figname_per_worker)
    return "Simulation done!"

@app.route('/apidbase/flaskapi/finalacgraph')
def finalacgraph():
    figname_per_worker = "/flaskstatic/acgraph" + str(os.getpid()) + ".png"
    return render_template("simulate.html", bpass_image="/flaskstatic/ltspice_screenshot" + str(os.getpid()) + ".png", user_image = figname_per_worker)

# main driver function
if __name__ == '__main__':

	# run() method of Flask class runs the application
	# on the local development server.
	app.run(host='0.0.0.0')

